 #!/usr/bin/env python
# -*- coding: utf-8 -*-

from os import environ
import platform
from twisted.internet import reactor, defer
import logging

from agent.SerialLineReceiver import SerialLineReceiver


def get_context():
    system = platform.system()
    if system == "Windows":
        from agent.windows.WindowsContext import WindowsContext as Context
    elif system == "Linux":
        from agent.linux.LinuxContext import LinuxContext as Context
    elif system == "FreeBSD":
        from agent.freebsd.FreeBSDContext import FreeBSDContext as Context
    else:
        raise NotImplementedError("Platform %s is not supported.", system)
    return Context


logging.basicConfig()
logger = logging.getLogger()
level = environ.get('LOGLEVEL', 'INFO')

logger.setLevel(level)


def init_serial():
    context = get_context()
    # Get proper serial class and port name
    (serial, port) = context.get_serial()
    logger.info("Opening port %s", port)
    # Open serial connection
    serial(SerialLineReceiver(context), port, reactor)
    try:
        from agent.notify import register_publisher
        register_publisher(reactor)
    except Exception:
        logger.exception("Could not register notify publisher")
    logger.debug("Starting reactor.")
    reactor.run()
    logger.debug("Reactor finished.")
