#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import join
import logging
from tarfile import TarFile, ReadError
from io import StringIO
from base64 import b64decode
from hashlib import md5
from datetime import datetime
import win32api
import wmi
import netifaces
from twisted.internet import reactor
from agent.windows.network import change_ip_windows
from agent.BaseContext import BaseContext

logger = logging.getLogger()
working_directory = r"C:\circle"  # noqa


class WindowsContext(BaseContext):

    @staticmethod
    def change_password(password):
        from win32com import adsi
        ads_obj = adsi.ADsGetObject('WinNT://localhost/%s,user' % 'cloud')
        ads_obj.Getinfo()
        ads_obj.SetPassword(password)

    @staticmethod
    def restart_networking():
        pass

    @staticmethod
    def change_ip(interfaces, dns):
        nameservers = WindowsContext._parse_nameserver(dns)
        change_ip_windows(interfaces, nameservers)

    @staticmethod
    def set_time(time):
        t = datetime.utcfromtimestamp(float(time))
        win32api.SetSystemTime(t.year, t.month, 0, t.day, t.hour,
                               t.minute, t.second, 0)

    @staticmethod
    def set_hostname(hostname):
        wmi.WMI().Win32_ComputerSystem()[0].Rename(hostname)

    @staticmethod
    def mount_store(host, username, password):
        from agent import notify
        url = 'cifs://%s:%s@%s/%s' % (username, password, host, username)
        for c in notify.clients:
            logger.debug("sending url %s to client %s", url, str(c))
            c.sendLine(url.encode())

    @staticmethod
    def get_keys():
        pass

    @staticmethod
    def add_keys(keys):
        pass

    @staticmethod
    def del_keys(keys):
        pass

    @staticmethod
    def cleanup():
        # TODO
        pass

    @staticmethod
    def start_access_server():
        # TODO
        pass

    @staticmethod
    def append(data, filename, chunk_number, uuid):
        if chunk_number == 0:
            flag = "w"
        else:
            flag = "a"
        with open(filename, flag) as myfile:
            myfile.write(data)

    @staticmethod
    def _update_registry(dir, executable):
        # HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\services\circle-agent
        from winreg import (OpenKeyEx, SetValueEx, QueryValueEx,
                             HKEY_LOCAL_MACHINE, KEY_ALL_ACCESS)
        with OpenKeyEx(HKEY_LOCAL_MACHINE,
                       r'SYSTEM\CurrentControlSet\services\circle-agent',
                       0,
                       KEY_ALL_ACCESS) as key:
            (old_executable, reg_type) = QueryValueEx(key, "ImagePath")
            SetValueEx(key, "ImagePath", None, 2, join(dir, executable))
        return old_executable

    @staticmethod
    def update(filename, executable, checksum, uuid):
        with open(filename, "r") as f:
            data = f.read()
            local_checksum = md5(data).hexdigest()
            if local_checksum != checksum:
                raise Exception("Checksum missmatch the file is damaged.")
            decoded = StringIO(b64decode(data))
        try:
            tar = TarFile.open("dummy", fileobj=decoded, mode='r|gz')
            tar.extractall(working_directory)
        except ReadError as e:
            logger.error(e)
        logger.info("Transfer completed!")
        WindowsContext._update_registry(working_directory, executable)
        logger.info('Updated')
        reactor.stop()

    @staticmethod
    def ipaddresses():
        args = {}
        interfaces = netifaces.interfaces()
        for i in interfaces:
            if i == 'lo':
                continue
            args[i] = []
            addresses = netifaces.ifaddresses(i)
            args[i] = ([x['addr']
                        for x in addresses.get(netifaces.AF_INET, [])] +
                       [x['addr']
                        for x in addresses.get(netifaces.AF_INET6, [])
                        if '%' not in x['addr']])
        return args

    @staticmethod
    def get_agent_version():
        try:
            with open(join(working_directory, 'version.txt')) as f:
                return f.readline()
        except IOError:
            return None

    @staticmethod
    def _get_virtio_device():
        path = None
        GUID = '{6FDE7521-1B65-48ae-B628-80BE62016026}'
        from infi.devicemanager import DeviceManager
        dm = DeviceManager()
        dm.root.rescan()
        # Search Virtio-Serial by name TODO: search by class_guid
        for i in dm.all_devices:
            if i.has_property("description"):
                if "virtio-serial".upper() in i.description.upper():
                    path = ("\\\\?\\" +
                            i.children[0].instance_id.lower().replace('\\', '#') +
                            "#" + GUID.lower()
                            )
        return path

    @staticmethod
    def get_serial():
        port = WindowsContext._get_virtio_device()
        import pythoncom
        pythoncom.CoInitialize()
        if port:
            from agent.windows.win32virtio import SerialPort
        else:
            from twisted.internet.serialport import SerialPort
            port = r'\\.\COM1'
        return (SerialPort, port)
